@@ -34,7 +34,9 @@ class OutgoingMessageHandler(Handler):
         self.delegate = delegate
 
     def on_link_flow(self, event):
-        if event.link.is_sender and event.link.credit:
+        if event.link.is_sender and event.link.credit \
+           and event.link.state & Endpoint.LOCAL_ACTIVE \
+           and event.link.state & Endpoint.REMOTE_ACTIVE :
             self.on_sendable(event)
 
     def on_delivery(self, event):
@@ -56,21 +58,21 @@ class OutgoingMessageHandler(Handler):
         Called when the sender link has credit and messages can
         therefore be transferred.
         """
-        if self.delegate:
+        if self.delegate != None:
             dispatch(self.delegate, 'on_sendable', event)
 
     def on_accepted(self, event):
         """
         Called when the remote peer accepts an outgoing message.
         """
-        if self.delegate:
+        if self.delegate != None:
             dispatch(self.delegate, 'on_accepted', event)
 
     def on_rejected(self, event):
         """
         Called when the remote peer rejects an outgoing message.
         """
-        if self.delegate:
+        if self.delegate != None:
             dispatch(self.delegate, 'on_rejected', event)
 
     def on_released(self, event):
@@ -79,7 +81,7 @@ class OutgoingMessageHandler(Handler):
         that this may be in response to either the RELEASE or MODIFIED
         state as defined by the AMQP specification.
         """
-        if self.delegate:
+        if self.delegate != None:
             dispatch(self.delegate, 'on_released', event)
 
     def on_settled(self, event):
@@ -88,7 +90,7 @@ class OutgoingMessageHandler(Handler):
         message. This is the point at which it shouod never be
         retransmitted.
         """
-        if self.delegate:
+        if self.delegate != None:
             dispatch(self.delegate, 'on_settled', event)
 
 def recv_msg(delivery):
@@ -182,11 +184,11 @@ class IncomingMessageHandler(Handler, Acking):
         explicitly accepting it, the ``delivery`` should be used, also
         obtainable via a property on the event.
         """
-        if self.delegate:
+        if self.delegate != None:
             dispatch(self.delegate, 'on_message', event)
 
     def on_settled(self, event):
-        if self.delegate:
+        if self.delegate != None:
             dispatch(self.delegate, 'on_settled', event)
 
 class EndpointStateHandler(Handler):
@@ -229,7 +231,7 @@ class EndpointStateHandler(Handler):
     @classmethod
     def print_error(cls, endpoint, endpoint_type):
         if endpoint.remote_condition:
-            logging.error(endpoint.remote_condition.description)
+            logging.error(endpoint.remote_condition.description or endpoint.remote_condition.name)
         elif cls.is_local_open(endpoint) and cls.is_remote_closed(endpoint):
             logging.error("%s closed by peer" % endpoint_type)
 
@@ -294,75 +296,75 @@ class EndpointStateHandler(Handler):
             event.link.open()
 
     def on_connection_opened(self, event):
-        if self.delegate:
+        if self.delegate != None:
             dispatch(self.delegate, 'on_connection_opened', event)
 
     def on_session_opened(self, event):
-        if self.delegate:
+        if self.delegate != None:
             dispatch(self.delegate, 'on_session_opened', event)
 
     def on_link_opened(self, event):
-        if self.delegate:
+        if self.delegate != None:
             dispatch(self.delegate, 'on_link_opened', event)
 
     def on_connection_opening(self, event):
-        if self.delegate:
+        if self.delegate != None:
             dispatch(self.delegate, 'on_connection_opening', event)
 
     def on_session_opening(self, event):
-        if self.delegate:
+        if self.delegate != None:
             dispatch(self.delegate, 'on_session_opening', event)
 
     def on_link_opening(self, event):
-        if self.delegate:
+        if self.delegate != None:
             dispatch(self.delegate, 'on_link_opening', event)
 
     def on_connection_error(self, event):
-        if self.delegate:
+        if self.delegate != None:
             dispatch(self.delegate, 'on_connection_error', event)
         else:
             self.log_error(event.connection, "connection")
 
     def on_session_error(self, event):
-        if self.delegate:
+        if self.delegate != None:
             dispatch(self.delegate, 'on_session_error', event)
         else:
             self.log_error(event.session, "session")
             event.connection.close()
 
     def on_link_error(self, event):
-        if self.delegate:
+        if self.delegate != None:
             dispatch(self.delegate, 'on_link_error', event)
         else:
             self.log_error(event.link, "link")
             event.connection.close()
 
     def on_connection_closed(self, event):
-        if self.delegate:
+        if self.delegate != None:
             dispatch(self.delegate, 'on_connection_closed', event)
 
     def on_session_closed(self, event):
-        if self.delegate:
+        if self.delegate != None:
             dispatch(self.delegate, 'on_session_closed', event)
 
     def on_link_closed(self, event):
-        if self.delegate:
+        if self.delegate != None:
             dispatch(self.delegate, 'on_link_closed', event)
 
     def on_connection_closing(self, event):
-        if self.delegate:
+        if self.delegate != None:
             dispatch(self.delegate, 'on_connection_closing', event)
         elif self.peer_close_is_error:
             self.on_connection_error(event)
 
     def on_session_closing(self, event):
-        if self.delegate:
+        if self.delegate != None:
             dispatch(self.delegate, 'on_session_closing', event)
         elif self.peer_close_is_error:
             self.on_session_error(event)
 
     def on_link_closing(self, event):
-        if self.delegate:
+        if self.delegate != None:
             dispatch(self.delegate, 'on_link_closing', event)
         elif self.peer_close_is_error:
             self.on_link_error(event)
@@ -371,7 +373,7 @@ class EndpointStateHandler(Handler):
         self.on_transport_closed(event)
 
     def on_transport_closed(self, event):
-        if self.delegate and event.connection and self.is_local_open(event.connection):
+        if self.delegate != None and event.connection and self.is_local_open(event.connection):
             dispatch(self.delegate, 'on_disconnected', event)
 
 class MessagingHandler(Handler, Acking):
