@@ -132,6 +132,9 @@ class Reactor(Wrapper):
         self.start()
         while self.process(): pass
         self.stop()
+        self.process()
+        self.global_handler = None
+        self.handler = None
 
     def wakeup(self):
         n = pn_reactor_wakeup(self._impl)
@@ -176,11 +179,41 @@ class Reactor(Wrapper):
             raise IOError("%s (%s:%s)" % (pn_error_text(pn_io_error(pn_reactor_io(self._impl))), host, port))
 
     def connection(self, handler=None):
+        """Deprecated: use connection_to_host() instead
+        """
         impl = _chandler(handler, self.on_error)
         result = Connection.wrap(pn_reactor_connection(self._impl, impl))
-        pn_decref(impl)
+        if impl: pn_decref(impl)
         return result
 
+    def connection_to_host(self, host, port, handler=None):
+        """Create an outgoing Connection that will be managed by the reactor.
+        The reator's pn_iohandler will create a socket connection to the host
+        once the connection is opened.
+        """
+        conn = self.connection(handler)
+        self.set_connection_host(conn, host, port)
+        return conn
+
+    def set_connection_host(self, connection, host, port):
+        """Change the address used by the connection.  The address is
+        used by the reactor's iohandler to create an outgoing socket
+        connection.  This must be set prior to opening the connection.
+        """
+        pn_reactor_set_connection_host(self._impl,
+                                       connection._impl,
+                                       unicode2utf8(str(host)),
+                                       unicode2utf8(str(port)))
+
+    def get_connection_address(self, connection):
+        """This may be used to retrieve the remote peer address.
+        @return: string containing the address in URL format or None if no
+        address is available.  Use the proton.Url class to create a Url object
+        from the returned value.
+        """
+        _url = pn_reactor_get_connection_address(self._impl, connection._impl)
+        return utf82unicode(_url)
+
     def selectable(self, handler=None):
         impl = _chandler(handler, self.on_error)
         result = Selectable.wrap(pn_reactor_selectable(self._impl))
@@ -273,7 +306,7 @@ class ApplicationEvent(EventBase):
 
     def __repr__(self):
         objects = [self.connection, self.session, self.link, self.delivery, self.subject]
-        return "%s(%s)" % (typename, ", ".join([str(o) for o in objects if o is not None]))
+        return "%s(%s)" % (self.type, ", ".join([str(o) for o in objects if o is not None]))
 
 class Transaction(object):
     """
@@ -497,34 +530,48 @@ class Connector(Handler):
         self.ssl_domain = None
         self.allow_insecure_mechs = True
         self.allowed_mechs = None
+        self.sasl_enabled = True
+        self.user = None
+        self.password = None
+        self.virtual_host = None
 
-    def _connect(self, connection):
+    def _connect(self, connection, reactor):
+        assert(reactor is not None)
         url = self.address.next()
-        # IoHandler uses the hostname to determine where to try to connect to
-        connection.hostname = "%s:%s" % (url.host, url.port)
-        logging.info("connecting to %s..." % connection.hostname)
+        reactor.set_connection_host(connection, url.host, str(url.port))
+        # if virtual-host not set, use host from address as default
+        if self.virtual_host is None:
+            connection.hostname = url.host
+        logging.debug("connecting to %s..." % url)
 
-        if url.username:
-            connection.user = url.username
-        if url.password:
-            connection.password = url.password
         transport = Transport()
-        sasl = transport.sasl()
-        sasl.allow_insecure_mechs = self.allow_insecure_mechs
-        if self.allowed_mechs:
-            sasl.allowed_mechs(self.allowed_mechs)
+        if self.sasl_enabled:
+            sasl = transport.sasl()
+            sasl.allow_insecure_mechs = self.allow_insecure_mechs
+            if url.username:
+                connection.user = url.username
+            elif self.user:
+                connection.user = self.user
+            if url.password:
+                connection.password = url.password
+            elif self.password:
+                connection.password = self.password
+            if self.allowed_mechs:
+                sasl.allowed_mechs(self.allowed_mechs)
         transport.bind(connection)
         if self.heartbeat:
             transport.idle_timeout = self.heartbeat
-        if url.scheme == 'amqps' and self.ssl_domain:
+        if url.scheme == 'amqps':
+            if not self.ssl_domain:
+                raise SSLUnavailable("amqps: SSL libraries not found")
             self.ssl = SSL(transport, self.ssl_domain)
             self.ssl.peer_hostname = url.host
 
     def on_connection_local_open(self, event):
-        self._connect(event.connection)
+        self._connect(event.connection, event.reactor)
 
     def on_connection_remote_open(self, event):
-        logging.info("connected to %s" % event.connection.hostname)
+        logging.debug("connected to %s" % event.connection.hostname)
         if self.reconnect:
             self.reconnect.reset()
             self.transport = None
@@ -539,16 +586,16 @@ class Connector(Handler):
                 delay = self.reconnect.next()
                 if delay == 0:
                     logging.info("Disconnected, reconnecting...")
-                    self._connect(self.connection)
+                    self._connect(self.connection, event.reactor)
                 else:
                     logging.info("Disconnected will try to reconnect after %s seconds" % delay)
                     event.reactor.schedule(delay, self)
             else:
-                logging.info("Disconnected")
+                logging.debug("Disconnected")
                 self.connection = None
 
     def on_timer_task(self, event):
-        self._connect(self.connection)
+        self._connect(self.connection, event.reactor)
 
     def on_connection_remote_close(self, event):
         self.connection = None
@@ -620,19 +667,64 @@ class Container(Reactor):
             self.container_id = str(generate_uuid())
             self.allow_insecure_mechs = True
             self.allowed_mechs = None
+            self.sasl_enabled = True
+            self.user = None
+            self.password = None
             Wrapper.__setattr__(self, 'subclass', self.__class__)
 
-    def connect(self, url=None, urls=None, address=None, handler=None, reconnect=None, heartbeat=None, ssl_domain=None):
+    def connect(self, url=None, urls=None, address=None, handler=None, reconnect=None, heartbeat=None, ssl_domain=None, **kwargs):
         """
         Initiates the establishment of an AMQP connection. Returns an
         instance of proton.Connection.
+
+        @param url: URL string of process to connect to
+
+        @param urls: list of URL strings of process to try to connect to
+
+        Only one of url or urls should be specified.
+
+        @param reconnect: A value of False will prevent the library
+        form automatically trying to reconnect if the underlying
+        socket is disconnected before the connection has been closed.
+
+        @param heartbeat: A value in milliseconds indicating the
+        desired frequency of heartbeats used to test the underlying
+        socket is alive.
+
+        @param ssl_domain: SSL configuration in the form of an
+        instance of proton.SSLdomain.
+
+        @param handler: a connection scoped handler that will be
+        called to process any events in the scope of this connection
+        or its child links
+
+        @param kwargs: sasl_enabled, which determines whether a sasl layer is
+        used for the connection; allowed_mechs an optional list of SASL
+        mechanisms to allow if sasl is enabled; allow_insecure_mechs a flag
+        indicating whether insecure mechanisms, such as PLAIN over a
+        non-encrypted socket, are allowed; 'virtual_host' the hostname to set
+        in the Open performative used by peer to determine the correct
+        back-end service for the client. If 'virtual_host' is not supplied the
+        host field from the URL is used instead."
+
         """
         conn = self.connection(handler)
         conn.container = self.container_id or str(generate_uuid())
+        conn.offered_capabilities = kwargs.get('offered_capabilities')
+        conn.desired_capabilities = kwargs.get('desired_capabilities')
+        conn.properties = kwargs.get('properties')
 
         connector = Connector(conn)
-        connector.allow_insecure_mechs = self.allow_insecure_mechs
-        connector.allowed_mechs = self.allowed_mechs
+        connector.allow_insecure_mechs = kwargs.get('allow_insecure_mechs', self.allow_insecure_mechs)
+        connector.allowed_mechs = kwargs.get('allowed_mechs', self.allowed_mechs)
+        connector.sasl_enabled = kwargs.get('sasl_enabled', self.sasl_enabled)
+        connector.user = kwargs.get('user', self.user)
+        connector.password = kwargs.get('password', self.password)
+        connector.virtual_host = kwargs.get('virtual_host')
+        if connector.virtual_host:
+            # only set hostname if virtual-host is a non-empty string
+            conn.hostname = connector.virtual_host
+
         conn._overrides = connector
         if url: connector.address = Urls([url])
         elif urls: connector.address = Urls(urls)
@@ -644,6 +736,8 @@ class Container(Reactor):
             connector.reconnect = reconnect
         elif reconnect is None:
             connector.reconnect = Backoff()
+        # use container's default client domain if none specified.  This is
+        # only necessary of the URL specifies the "amqps:" scheme
         connector.ssl_domain = ssl_domain or (self.ssl and self.ssl.client)
         conn._session_policy = SessionPerConnection() #todo: make configurable
         conn.open()
@@ -700,7 +794,7 @@ class Container(Reactor):
             snd.source.address = source
         if target:
             snd.target.address = target
-        if handler:
+        if handler != None:
             snd.handler = handler
         if tags:
             snd.tag_generator = tags
@@ -743,7 +837,7 @@ class Container(Reactor):
             rcv.source.dynamic = True
         if target:
             rcv.target.address = target
-        if handler:
+        if handler != None:
             rcv.handler = handler
         _apply_link_options(options, rcv)
         rcv.open()
@@ -772,8 +866,12 @@ class Container(Reactor):
         url = Url(url)
         acceptor = self.acceptor(url.host, url.port)
         ssl_config = ssl_domain
-        if not ssl_config and url.scheme == 'amqps' and self.ssl:
-            ssl_config = self.ssl.server
+        if not ssl_config and url.scheme == 'amqps':
+            # use container's default server domain
+            if self.ssl:
+                ssl_config = self.ssl.server
+            else:
+                raise SSLUnavailable("amqps: SSL libraries not found")
         if ssl_config:
             acceptor.set_ssl_domain(ssl_config)
         return acceptor
