@@ -174,7 +174,13 @@ typedef enum {
    * An AMQP map. A polymorphic container of other AMQP values formed
    * into key/value pairs.
    */
-  PN_MAP = 25
+  PN_MAP = 25,
+
+  /**
+   * A special invalid type value that is returned when no valid type
+   * is available.
+   */
+  PN_INVALID = -1
 } pn_type_t;
 
 /**
@@ -468,8 +474,8 @@ PN_EXTERN bool pn_data_exit(pn_data_t *data);
 PN_EXTERN bool pn_data_lookup(pn_data_t *data, const char *name);
 
 /**
- * Access the type of the current node. Returns an undefined value if
- * there is no current node.
+ * Access the type of the current node. Returns PN_INVALID if there is no
+ * current node.
  *
  * @param data a data object
  * @return the type of the current node
@@ -506,8 +512,7 @@ PN_EXTERN int pn_data_format(pn_data_t *data, char *bytes, size_t *size);
  * @param bytes the buffer for encoded data
  * @param size the size of the buffer
  *
- * @param ssize_t returns the size of the encoded data on success or
- *                an error code on failure
+ * @return the size of the encoded data on success or an error code on failure
  */
 PN_EXTERN ssize_t pn_data_encode(pn_data_t *data, char *bytes, size_t size);
 
@@ -516,7 +521,7 @@ PN_EXTERN ssize_t pn_data_encode(pn_data_t *data, char *bytes, size_t size);
  *
  * @param data the data object
  *
- * @param ssize_t returns the size of the encoded data or an error code if data is invalid.
+ * @return the size of the encoded data or an error code if data is invalid.
  */
 PN_EXTERN ssize_t pn_data_encoded_size(pn_data_t *data);
 
@@ -542,7 +547,7 @@ PN_EXTERN ssize_t pn_data_decode(pn_data_t *data, const char *bytes, size_t size
 /**
  * Puts an empty list value into a pn_data_t. Elements may be filled
  * by entering the list node using ::pn_data_enter() and using
- * ::pn_data_put_* to add the desired contents. Once done,
+ * pn_data_put_* to add the desired contents. Once done,
  * ::pn_data_exit() may be used to return to the current level in the
  * tree and put more values.
  *
@@ -912,7 +917,7 @@ PN_EXTERN size_t pn_data_get_map(pn_data_t *data);
  * If the current node is an array, return the number of elements in
  * the array, otherwise return 0. Array data can be accessed by
  * entering the array. If the array is described, the first child node
- * will be the descriptor, and the remaining @var count child nodes
+ * will be the descriptor, and the remaining count child nodes
  * will be the elements of the array.
  *
  * @code
@@ -953,7 +958,7 @@ PN_EXTERN bool pn_data_is_array_described(pn_data_t *data);
 
 /**
  * Return the array type if the current node points to an array,
- * undefined otherwise.
+ * PN_INVALID otherwise.
  *
  * @param data a pn_data_t object
  * @return the element type of an array node
@@ -1246,7 +1251,7 @@ PN_EXTERN pn_handle_t pn_data_point(pn_data_t *data);
  * otherwise it will return true.
  *
  * @param data a pn_data_t object
- * @param handle a handle referencing the saved navigational state
+ * @param point a handle referencing the saved navigational state
  * @return true iff the prior navigational state was restored
  */
 PN_EXTERN bool pn_data_restore(pn_data_t *data, pn_handle_t point);
