@@ -52,6 +52,7 @@ extern "C" {
  * - ::PN_TRACE_RAW
  * - ::PN_TRACE_FRM
  * - ::PN_TRACE_DRV
+ * - ::PN_TRACE_EVT
  *
  */
 typedef int pn_trace_t;
@@ -81,6 +82,11 @@ typedef void (*pn_tracer_t)(pn_transport_t *transport, const char *message);
  */
 #define PN_TRACE_DRV (4)
 
+/**
+ * Log events
+ */
+#define PN_TRACE_EVT (8)
+
 /**
  * Factory for creating a transport.
  * A transport is used by a connection to interface with the network.
