@@ -1,6 +1,7 @@
 #ifndef PROTON_TYPES_H
 #define PROTON_TYPES_H 1
 
+
 /*
  *
  * Licensed to the Apache Software Foundation (ASF) under one
@@ -46,7 +47,13 @@ typedef int32_t  pn_sequence_t;
 typedef uint32_t pn_millis_t;
 #define PN_MILLIS_MAX (~0U)
 typedef uint32_t pn_seconds_t;
+
 typedef int64_t  pn_timestamp_t;
+
+/** Return a timestamp for the time now. */
+PN_EXTERN pn_timestamp_t pn_timestamp_now(void);
+
+
 typedef uint32_t pn_char_t;
 typedef uint32_t pn_decimal32_t;
 typedef uint64_t pn_decimal64_t;
@@ -255,6 +262,15 @@ typedef struct pn_collector_t pn_collector_t;
 
 typedef struct pn_transport_t pn_transport_t;
 
+/**
+ * An event handler
+ *
+ * A pn_handler_t is target of ::pn_event_t dispatched by the ::pn_reactor_t
+ *
+ * @ingroup reactor
+ */
+typedef struct pn_handler_t pn_handler_t;
+
 /** @}
  */
 #ifdef __cplusplus
