@@ -68,6 +68,7 @@ const char *pn_type_name(pn_type_t type)
   case PN_ARRAY: return "PN_ARRAY";
   case PN_LIST: return "PN_LIST";
   case PN_MAP: return "PN_MAP";
+  default: break;
   }
 
   return "<UNKNOWN>";
@@ -265,7 +266,7 @@ int pni_inspect_enter(void *ctx, pn_data_t *data, pni_node_t *node)
       return 0;
     }
     const char *name = (index < grandfields->field_count)
-        ? FIELD_STRINGPOOL+FIELD_FIELDS[grandfields->first_field_index+index]
+        ? FIELD_STRINGPOOL.STRING0+FIELD_FIELDS[grandfields->first_field_index+index]
         : NULL;
     if (name) {
       err = pn_string_addf(str, "%s=", name);
@@ -285,7 +286,7 @@ int pni_inspect_enter(void *ctx, pn_data_t *data, pni_node_t *node)
     return pn_string_addf(str, "{");
   default:
     if (fields && index == 0) {
-      err = pn_string_addf(str, "%s", FIELD_STRINGPOOL+FIELD_NAME[fields->name_index]);
+      err = pn_string_addf(str, "%s", FIELD_STRINGPOOL.STRING0+FIELD_NAME[fields->name_index]);
       if (err) return err;
       err = pn_string_addf(str, "(");
       if (err) return err;
@@ -417,10 +418,14 @@ void pn_data_clear(pn_data_t *data)
 
 static int pni_data_grow(pn_data_t *data)
 {
-  pni_nid_t new_capacity = 2 * (data->capacity ? data->capacity : 2);
-  pni_node_t *new_nodes = (pni_node_t *)realloc(data->nodes, new_capacity * sizeof(pni_node_t));
+  size_t capacity = data->capacity ? data->capacity : 2;
+  if (capacity >= PNI_NID_MAX) return PN_OUT_OF_MEMORY;
+  else if (capacity < PNI_NID_MAX/2) capacity *= 2;
+  else capacity = PNI_NID_MAX;
+
+  pni_node_t *new_nodes = (pni_node_t *)realloc(data->nodes, capacity * sizeof(pni_node_t));
   if (new_nodes == NULL) return PN_OUT_OF_MEMORY;
-  data->capacity = new_capacity;
+  data->capacity = capacity;
   data->nodes = new_nodes;
   return 0;
 }
@@ -693,7 +698,7 @@ static bool pn_scan_next(pn_data_t *data, pn_type_t *type, bool suspend)
       pn_data_exit(data);
       return pn_scan_next(data, type, suspend);
     } else {
-      *type = (pn_type_t) -1;
+      *type = PN_INVALID;
       return false;
     }
   }
@@ -1148,9 +1153,9 @@ void pn_data_widen(pn_data_t *data)
 pn_handle_t pn_data_point(pn_data_t *data)
 {
   if (data->current) {
-    return data->current;
+    return (pn_handle_t)(uintptr_t)data->current;
   } else {
-    return -data->parent;
+    return (pn_handle_t)(uintptr_t)-data->parent;
   }
 }
 
@@ -1161,8 +1166,8 @@ bool pn_data_restore(pn_data_t *data, pn_handle_t point)
     data->parent = -((pn_shandle_t) point);
     data->current = 0;
     return true;
-  } else if (point && point <= data->size) {
-    data->current = point;
+  } else if (spoint && spoint <= data->size) {
+    data->current = spoint;
     pni_node_t *current = pni_data_current(data);
     data->parent = current->parent;
     return true;
@@ -1267,7 +1272,7 @@ pn_type_t pn_data_type(pn_data_t *data)
   if (node) {
     return node->atom.type;
   } else {
-    return (pn_type_t) -1;
+    return PN_INVALID;
   }
 }
 
@@ -1277,7 +1282,7 @@ pn_type_t pni_data_parent_type(pn_data_t *data)
   if (node) {
     return node->atom.type;
   } else {
-    return (pn_type_t) -1;
+    return PN_INVALID;
   }
 }
 
@@ -1719,7 +1724,7 @@ pn_type_t pn_data_get_array_type(pn_data_t *data)
   if (node && node->atom.type == PN_ARRAY) {
     return node->type;
   } else {
-    return (pn_type_t) -1;
+    return PN_INVALID;
   }
 }
 
@@ -2124,6 +2129,8 @@ int pn_data_appendn(pn_data_t *data, pn_data_t *src, int limit)
       pn_data_enter(src);
       level++;
       break;
+    default:
+      break;
     }
 
     if (err) { pn_data_restore(src, point); return err; }
