@@ -106,7 +106,7 @@ struct pn_messenger_t {
   int draining;      // # links in drain state
   int connection_error;
   int flags;
-  pn_snd_settle_mode_t snd_settle_mode;
+  int snd_settle_mode;          /* pn_snd_settle_mode_t or -1 for unset */
   pn_rcv_settle_mode_t rcv_settle_mode;
   pn_tracer_t tracer;
   pn_ssl_verify_mode_t ssl_peer_authentication_mode;
@@ -665,7 +665,7 @@ pn_messenger_t *pn_messenger(const char *name)
     m->domain = pn_string(NULL);
     m->connection_error = 0;
     m->flags = PN_FLAGS_ALLOW_INSECURE_MECHS; // TODO: Change this back to 0 for the Proton 0.11 release
-    m->snd_settle_mode = PN_SND_SETTLED;
+    m->snd_settle_mode = -1;    /* Default depends on sender/receiver */
     m->rcv_settle_mode = PN_RCV_FIRST;
     m->tracer = NULL;
     m->ssl_peer_authentication_mode = PN_SSL_VERIFY_PEER_NAME;
@@ -931,7 +931,7 @@ static int pn_transport_config(pn_messenger_t *messenger,
     pn_transport_set_tracer(transport, messenger->tracer);
   if (ctx->scheme && !strcmp(ctx->scheme, "amqps")) {
     pn_ssl_domain_t *d = pn_ssl_domain(PN_SSL_MODE_CLIENT);
-    if (messenger->certificate && messenger->private_key) {
+    if (messenger->certificate) {
       int err = pn_ssl_domain_set_credentials( d, messenger->certificate,
                                                messenger->private_key,
                                                messenger->password);
@@ -1495,7 +1495,7 @@ int pn_messenger_start(pn_messenger_t *messenger)
   // returned. Currently no attempt is made to check the name part of the
   // address, as the intent here is to fail fast if the addressed host
   // is invalid or unavailable.
-  if (messenger->flags | PN_FLAGS_CHECK_ROUTES) {
+  if (messenger->flags & PN_FLAGS_CHECK_ROUTES) {
     pn_list_t *substitutions = pn_list(PN_WEAKREF, 0);
     pn_transform_get_substitutions(messenger->routes, substitutions);
     for (size_t i = 0; i < pn_list_size(substitutions) && error == 0; i++) {
@@ -1699,7 +1699,7 @@ pn_connection_t *pn_messenger_resolve(pn_messenger_t *messenger, const char *add
   return connection;
 }
 
-PN_EXTERN pn_link_t *pn_messenger_get_link(pn_messenger_t *messenger,
+pn_link_t *pn_messenger_get_link(pn_messenger_t *messenger,
                                            const char *address, bool sender)
 {
   char *name = NULL;
@@ -1748,11 +1748,17 @@ pn_link_t *pn_messenger_link(pn_messenger_t *messenger, const char *address,
 
   if ((sender && pn_messenger_get_outgoing_window(messenger)) ||
       (!sender && pn_messenger_get_incoming_window(messenger))) {
-    // use required settlement (defaults to sending pre-settled messages)
-    pn_link_set_snd_settle_mode(link, messenger->snd_settle_mode);
+    if (messenger->snd_settle_mode == -1) { /* Choose default based on sender/receiver */
+      /* For a sender use MIXED so the application can decide whether each
+         message is settled or not. For a receiver request UNSETTLED, since the
+         user set an incoming_window which means they want to decide settlement.
+      */
+      pn_link_set_snd_settle_mode(link, sender ? PN_SND_MIXED : PN_SND_UNSETTLED);
+    } else {                    /* Respect user setting */
+      pn_link_set_snd_settle_mode(link, (pn_snd_settle_mode_t)messenger->snd_settle_mode);
+    }
     pn_link_set_rcv_settle_mode(link, messenger->rcv_settle_mode);
   }
-  // XXX
   if (pn_streq(name, "#")) {
     if (pn_link_is_sender(link)) {
       pn_terminus_set_dynamic(pn_link_target(link), true);
@@ -2286,7 +2292,7 @@ int pn_messenger_reject(pn_messenger_t *messenger, pn_tracker_t tracker, int fla
                           PN_STATUS_REJECTED, flags, false, false);
 }
 
-PN_EXTERN pn_link_t *pn_messenger_tracker_link(pn_messenger_t *messenger,
+pn_link_t *pn_messenger_tracker_link(pn_messenger_t *messenger,
                                                pn_tracker_t tracker)
 {
   pni_store_t *store = pn_tracker_store(messenger, tracker);
@@ -2347,17 +2353,21 @@ int pn_messenger_rewrite(pn_messenger_t *messenger, const char *pattern, const c
   return 0;
 }
 
-PN_EXTERN int pn_messenger_set_flags(pn_messenger_t *messenger, const int flags)
+int pn_messenger_set_flags(pn_messenger_t *messenger, const int flags)
 {
   if (!messenger)
     return PN_ARG_ERR;
-  if (flags != 0 && (flags ^ PN_FLAGS_CHECK_ROUTES) != 0)
+  if (flags == 0) {
+    messenger->flags = 0;
+  } else if (flags & (PN_FLAGS_CHECK_ROUTES | PN_FLAGS_ALLOW_INSECURE_MECHS)) {
+    messenger->flags |= flags;
+  } else {
     return PN_ARG_ERR;
-  messenger->flags = flags;
+  }
   return 0;
 }
 
-PN_EXTERN int pn_messenger_get_flags(pn_messenger_t *messenger)
+int pn_messenger_get_flags(pn_messenger_t *messenger)
 {
   return messenger ? messenger->flags : 0;
 }
