@@ -23,8 +23,12 @@
  */
 
 #include <proton/reactor.h>
+#include <proton/url.h>
 
 void pni_record_init_reactor(pn_record_t *record, pn_reactor_t *reactor);
-
+void pni_event_set_root(pn_event_t *event, pn_handler_t *handler);
+void pni_reactor_set_connection_peer_address(pn_connection_t *connection,
+                                             const char *host,
+                                             const char *port);
 
 #endif /* src/reactor.h */
