@@ -32,13 +32,13 @@
 #include <proton/types.h>
 #include <proton/object.h>
 
-PN_EXTERN void pni_parse_url(char *url, char **scheme, char **user, char **pass, char **host, char **port, char **path);
+void pni_parse_url(char *url, char **scheme, char **user, char **pass, char **host, char **port, char **path);
 void pni_fatal(const char *fmt, ...);
 void pni_vfatal(const char *fmt, va_list ap);
-PN_EXTERN ssize_t pn_quote_data(char *dst, size_t capacity, const char *src, size_t size);
+ssize_t pn_quote_data(char *dst, size_t capacity, const char *src, size_t size);
 int pn_quote(pn_string_t *dst, const char *src, size_t size);
-PN_EXTERN void pn_fprint_data(FILE *stream, const char *bytes, size_t size);
-PN_EXTERN void pn_print_data(const char *bytes, size_t size);
+void pn_fprint_data(FILE *stream, const char *bytes, size_t size);
+void pn_print_data(const char *bytes, size_t size);
 bool pn_env_bool(const char *name);
 pn_timestamp_t pn_timestamp_min(pn_timestamp_t a, pn_timestamp_t b);
 
