@@ -104,7 +104,7 @@ public class MessengerImpl implements Messenger
 
     /**
      * @deprecated This constructor's visibility will be reduced to the default scope in a future release.
-     * Client code outside this module should use a {@link MessengerFactory} instead
+     * Client code outside this module should use {@link Messenger.Factory#create()} instead
      */
     @Deprecated public MessengerImpl()
     {
@@ -113,7 +113,7 @@ public class MessengerImpl implements Messenger
 
     /**
      * @deprecated This constructor's visibility will be reduced to the default scope in a future release.
-     * Client code outside this module should use a {@link MessengerFactory} instead
+     * Client code outside this module should use a {@link Messenger.Factory#create(String)} instead
      */
     @Deprecated public MessengerImpl(String name)
     {
@@ -790,8 +790,13 @@ public class MessengerImpl implements Messenger
         for (Link link : new Links(connection, UNINIT, ANY))
         {
             //TODO: the following is not correct; should only copy those properties that we understand
-            link.setSource(link.getRemoteSource());
-            link.setTarget(link.getRemoteTarget());
+            //TODO: is this any better:
+            if (link.getRemoteSource() != null) {
+                link.setSource(link.getRemoteSource().copy());
+            }
+            if (link.getRemoteTarget() != null) {
+                link.setTarget(link.getRemoteTarget().copy());
+            }
             linkAdded(link);
             link.open();
             _logger.log(Level.FINE, "Opened link " + link);
@@ -1472,7 +1477,7 @@ public class MessengerImpl implements Messenger
         }
     }
 
-    private class ConnectionContext
+    private static class ConnectionContext
     {
         private Address _address;
         private Connector _connector;
