@@ -18,6 +18,15 @@
 #
 
 from unittest import TestCase
+try:
+  from unittest import SkipTest
+except:
+  try:
+    from unittest2 import SkipTest
+  except:
+    class SkipTest(Exception):
+      pass
+
 from random import randint
 from threading import Thread
 from socket import socket, AF_INET, SOCK_STREAM
@@ -105,6 +114,16 @@ def pump(transport1, transport2, buffer_size=1024):
          pump_uni(transport2, transport1, buffer_size)):
     pass
 
+def findfileinpath(filename, searchpath):
+    """Find filename in the searchpath
+        return absolute path to the file or None
+    """
+    paths = searchpath.split(os.pathsep)
+    for path in paths:
+        if os.path.exists(os.path.join(path, filename)):
+            return os.path.abspath(os.path.join(path, filename))
+    return None
+
 def isSSLPresent():
     return SSL.present()
 
@@ -116,6 +135,8 @@ def _cyrusSetup(conf_dir):
   saslpasswd = ""
   if 'SASLPASSWD' in os.environ:
     saslpasswd = os.environ['SASLPASSWD']
+  else:
+    saslpasswd = findfileinpath('saslpasswd2', os.getenv('PATH')) or ""
   if os.path.exists(saslpasswd):
     t = Template("""sasldb_path: ${db}
 mech_list: EXTERNAL DIGEST-MD5 SCRAM-SHA-1 CRAM-MD5 PLAIN ANONYMOUS
@@ -147,7 +168,11 @@ def ensureCanTestExtendedSASL():
   if not createdSASLDb:
     raise Skipped("Can't Test Extended SASL: Couldn't create auth db")
 
+class DefaultConfig:
+    defines = {}
+
 class Test(TestCase):
+  config = DefaultConfig()
 
   def __init__(self, name):
     super(Test, self).__init__(name)
@@ -176,7 +201,7 @@ class Test(TestCase):
     return int(self.default("verbose", 0))
 
 
-class Skipped(Exception):
+class Skipped(SkipTest):
   skipped = True
 
 
@@ -255,16 +280,6 @@ class MessengerApp(object):
         self.password = None
         self._output = None
 
-    def findfile(self, filename, searchpath):
-        """Find filename in the searchpath
-            return absolute path to the file or None
-        """
-        paths = string.split(searchpath, os.pathsep)
-        for path in paths:
-            if os.path.exists(os.path.join(path, filename)):
-                return os.path.abspath(os.path.join(path, filename))
-        return None
-
     def start(self, verbose=False):
         """ Begin executing the test """
         cmd = self.cmdline()
@@ -273,19 +288,17 @@ class MessengerApp(object):
             print("COMMAND='%s'" % str(cmd))
         #print("ENV='%s'" % str(os.environ.copy()))
         try:
-            if os.name=="nt":
-                # Windows handles python launch by replacing script 'filename' with
-                # 'python abspath-to-filename' in cmdline arg list.
-                if cmd[0].endswith('.py'):
-                    foundfile = self.findfile(cmd[0], os.environ['PATH'])
-                    if foundfile is None:
-                        foundfile = self.findfile(cmd[0], os.environ['PYTHONPATH'])
-                        msg = "Unable to locate file '%s' in PATH or PYTHONPATH" % cmd[0]
-                        raise Skipped("Skipping test - %s" % msg)
+            # Handle python launch by replacing script 'filename' with
+            # 'python abspath-to-filename' in cmdline arg list.
+            if cmd[0].endswith('.py'):
+                foundfile = findfileinpath(cmd[0], os.getenv('PATH'))
+                if foundfile is None:
+                    msg = "Unable to locate file '%s' in PATH" % cmd[0]
+                    raise Skipped("Skipping test - %s" % msg)
 
-                    del cmd[0:1]
-                    cmd.insert(0, foundfile)
-                    cmd.insert(0, sys.executable)
+                del cmd[0:1]
+                cmd.insert(0, foundfile)
+                cmd.insert(0, sys.executable)
             self._process = Popen(cmd, stdout=PIPE, stderr=STDOUT,
                                   bufsize=4096, universal_newlines=True)
         except OSError:
@@ -483,7 +496,7 @@ class MessengerSenderValgrind(MessengerSenderC):
         if not suppressions:
             suppressions = os.path.join(os.path.dirname(__file__),
                                         "valgrind.supp" )
-        self._command = [os.environ["VALGRIND"], "--error-exitcode=1", "--quiet",
+        self._command = [os.environ["VALGRIND"], "--error-exitcode=42", "--quiet",
                          "--trace-children=yes", "--leak-check=full",
                          "--suppressions=%s" % suppressions] + self._command
 
@@ -502,7 +515,7 @@ class MessengerReceiverValgrind(MessengerReceiverC):
         if not suppressions:
             suppressions = os.path.join(os.path.dirname(__file__),
                                         "valgrind.supp" )
-        self._command = [os.environ["VALGRIND"], "--error-exitcode=1", "--quiet",
+        self._command = [os.environ["VALGRIND"], "--error-exitcode=42", "--quiet",
                          "--trace-children=yes", "--leak-check=full",
                          "--suppressions=%s" % suppressions] + self._command
 
@@ -534,7 +547,7 @@ class ReactorSenderValgrind(ReactorSenderC):
         if not suppressions:
             suppressions = os.path.join(os.path.dirname(__file__),
                                         "valgrind.supp" )
-        self._command = [os.environ["VALGRIND"], "--error-exitcode=1", "--quiet",
+        self._command = [os.environ["VALGRIND"], "--error-exitcode=42", "--quiet",
                          "--trace-children=yes", "--leak-check=full",
                          "--suppressions=%s" % suppressions] + self._command
 
@@ -553,7 +566,6 @@ class ReactorReceiverValgrind(ReactorReceiverC):
         if not suppressions:
             suppressions = os.path.join(os.path.dirname(__file__),
                                         "valgrind.supp" )
-        self._command = [os.environ["VALGRIND"], "--error-exitcode=1", "--quiet",
+        self._command = [os.environ["VALGRIND"], "--error-exitcode=42", "--quiet",
                          "--trace-children=yes", "--leak-check=full",
                          "--suppressions=%s" % suppressions] + self._command
-
