@@ -76,11 +76,11 @@ class AppTests(Test):
     def valgrind_test(self):
         self.is_valgrind = True
 
-    def setup(self):
+    def setUp(self):
         self.senders = []
         self.receivers = []
 
-    def teardown(self):
+    def tearDown(self):
         pass
 
     def _do_test(self, iterations=1):
@@ -124,6 +124,8 @@ class MessengerTests(AppTests):
     def _ssl_check(self):
         if not isSSLPresent():
             raise Skipped("No SSL libraries found.")
+        if os.name=="nt":
+            raise Skipped("Windows SChannel lacks anonymous cipher support.")
 
     def __init__(self, *args):
         AppTests.__init__(self, *args)
