@@ -5,7 +5,7 @@ require 'qpid_proton'
 
 class SmokeTest < Test::Unit::TestCase
 
-  Messenger = Qpid::Proton::Messenger
+  Messenger = Qpid::Proton::Messenger::Messenger
   Message = Qpid::Proton::Message
 
   def setup
@@ -29,8 +29,8 @@ class SmokeTest < Test::Unit::TestCase
 
     pump()
 
-    assert @client.stopped
-    assert @server.stopped
+    assert @client.stopped?
+    assert @server.stopped?
   end
 
   def testSmoke(count=10)
